/*
 * Copyright (c) 1994 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

/* Class wrapper for dlg coverage object */

#ifndef dlgcov_h
#define dlgcov_h

#include <Map/mapcov.h>

class DlgFeatureClass;
class DlgLibrary;

class DlgCoverage : public MapCoverage {
public:
    DlgCoverage (DlgLibrary*, const char *coverage_name);
    ~DlgCoverage ();

    const char* path() const;
    DlgLibrary* library() const { return (DlgLibrary*)_library; }
    virtual void library(DlgLibrary* lib) { _library = (MapLibrary*)lib; }

    virtual const char* description() const;
    virtual void description(char*);

    DlgFeatureClass* feature_class(unsigned fclsnum) const;
    DlgFeatureClass* feature_class(const char*) const;

protected:
    virtual char** feature_class_names (int& nfcls) const;
    virtual MapFeatureClass* make_feature_class(const char* name);


    virtual void load_edge_table() {};
    virtual void load_node_table() {};
    virtual void load_face_table() {};

protected:
    char* _desc;

};

#endif /* defined(dlgcov_h) */
