#ifndef H_COORD_DEF

#define H_COORD_DEF

#if 0
#ifndef INCL_XVTH
#include "xvt.h"
#endif
#endif

#include "noxvt.h"

#include "math.h"

/***************************************************************
@    Misc. Constants
****************************************************************
*/
#define NORTH_HEMI               0
#define SOUTH_HEMI               1
#define EAST_HEMI                0
#define WEST_HEMI                1
#define USER_DEFINED             0
#define GET_USER_POINT           20000



/***************************************************************
@    ARCPOINT
****************************************************************
ARC map projection parameters
*/

typedef struct
{
    long            row;
    long            col;
}               ARCPOINT;
/*
 * Description:
 */

/***************************************************************
@    ASPECT
****************************************************************
Aspect of a map projection
*/
typedef struct
{
    GFLOAT          plon;
    GFLOAT          plat;
    GFLOAT          plat1;
    GFLOAT          plat2;
}               ASPECT;

/*
 * Description: Plon and plat are the points of tangency of the developable
 * surface.  Plat1 and Plat2 are the standard parallels for the Lambert
 * Conformal Conic.
 */

/***************************************************************
@  CONVERSION_TYPE
****************************************************************
*/
typedef struct
{
   int     from_ellips; /* CC1, DT. isph in FORTRAN; nnfr in NT MADTRAN */
   double  from_a;      /* CC1, DT */
   double  from_f;      /* CC1, DT */
   double  from_e_sq;   /* DT */
   int     to_ellips;   /* CC2. nnto in NT MADTRAN */
   double  to_a;        /* CC2 */
   double  to_f;        /* CC2 */
   double  da;          /* DT */
   double  df;          /* DT */
   int     dx;          /* DT */ /***DO!! change to double*/
   int     dy;          /* DT */
   int     dz;          /* DT */
   int     ex;          /* DT */
   int     ey;          /* DT */
   int     ez;          /* DT */
} CONVERSION_TYPE;
/*
Description: Structure used to store all parameters needed by
the transformation and conversion algorithms.  "from_" values
and "to_" values belong to the from ellipsoid and to ellipsoid
respectively. da and df are the deltas between the two ellipsoids'
semi-major axes and flattening values (to-ellipsoid minus from-
ellipsoid). Comments following each parameter indicate which algorithm
uses it: DT = datum transformation, CC1 = coordinate conversion on initial
point, and CC2 = coordinate conversion on transformed point.
*/


/* Can replace INPUT_ENUM and DD_FORMAT_ENUM in Package 1*/
/***************************************************************
@    COORD_ENUM
****************************************************************
*/
typedef enum
{
    COORD_GP_D,
    COORD_GP_DM,
    COORD_GP_DMS,
    COORD_UTM,
    COORD_UPS,
    COORD_MGRS,
    COORD_XYZ,
    COORD_AE,
    COORD_AKRSO,
    COORD_GEOREF,
    COORD_LAM,
    COORD_MADL,
    COORD_MER,
    COORD_MRSO,
    COORD_NZMG,
    COORD_OBM,
    COORD_TM
} COORD_ENUM;
/*
Description: User's choice for type of point coordinates.
*/

/***************************************************************
@    COORD_DM
****************************************************************
A geo-coordinate in deg, min, sec
*/
typedef struct
{
    short           d;		/* degrees */
    GFLOAT          m;		/* minutes */
}               COORD_DM;

/*
 * Description:
 */

/***************************************************************
@    COORD_DMS
****************************************************************
A geo-coordinate in deg, min, sec
*/
typedef struct
{
    short           d;		/* degrees */
    short           m;		/* minutes */
    GFLOAT          s;		/* seconds */
}               COORD_DMS;

/*
 * Description:
 */

/***************************************************************
@    GEOGRAPHIC
****************************************************************
Geographic coordinates are stored internally in DD.
*/
#undef SEC
typedef enum
{
    DD,
    DMS,
    GEOREF,
    MGRS,
    MIN,
    SEC,
    UTM
}
                GEOGRAPHIC;

/*
 * Description:
 */

/***************************************************************
@  GRID_PARAMS
****************************************************************
*/
typedef struct
{
    double a;
    double b;
    double e;
    double es;
    double fe;
    double fn;
    double oaz;
    double ogam;
    double ok;
    double olam;
    double slam;
    double ta;
    double tb;
    double te;
} GRID_PARAMS;
/*
 * Description:
 */

/***************************************************************
@    HOR_DATUM
****************************************************************
Horizontal datums
*/
typedef enum
{
    NDEFINED,
    HD_WGS84,
    HD_WGS72,
    HD_NAD27
}
                HOR_DATUM;

/*
 * Description:
 */

/***************************************************************
@    HOR_DISTANCE
****************************************************************
Horizontal distance is stored internally in meters.
*/
typedef enum
{
    HOR_METERS,
    HOR_FEET,
    HOR_KILOMETERS,
    HOR_LEAGUES,
    HOR_NAUTICAL_MILES,
    HOR_STATUTE_MILES,
    HOR_SEC_OF_ARC,
    HOR_YARDS
}
                HOR_DISTANCE;

/*
 * Description:
 * 
 */

/***************************************************************
@    LAMBER_PARAMS
****************************************************************
Lambert Conformal Conic map projection parameters
*/
typedef struct
{
    GFLOAT          n;
    GFLOAT          F;
    GFLOAT          rho0;
}               LAMBER_PARAMS;

/*
 * Description:
 */

/***************************************************************
@    LINEAR_PARAMS
****************************************************************
Linear (Platt-Carree) map projection parameters
*/
typedef struct
{
    GFLOAT          dummy;
}               LINEAR_PARAMS;

/*
 * Description:
 */

/***************************************************************
@    MERCAT_PARAMS
****************************************************************
Mercator map projection parameters
*/
typedef struct
{
    GFLOAT          dummy;
}               MERCAT_PARAMS;

/*
 * Description:
 */

/***************************************************************
@    POINT_DD
****************************************************************
A Point in decimal degrees
*/
typedef struct
{
    GFLOAT          lon;
    GFLOAT          lat;
}               POINT_DD;

/*
 * Description:
 */

/***************************************************************
@    POINT_XY
****************************************************************
Any point POINT_DD structure
*/

typedef struct
{
 	GFLOAT 		x;
	GFLOAT		y;
} 		POINT_XY;

/*
 * Description:
 */

/***************************************************************
@    POINT_XYZ
****************************************************************
A Point in decimal degrees
*/
typedef struct
{
    GFLOAT          x;
    GFLOAT          y;
    GFLOAT	    z;
}               POINT_XYZ;

/*
 * Description:
 */

/***************************************************************
@    POINT_DM
****************************************************************
A geo-coordinate in deg, min
*/
typedef struct
{
    COORD_DM     lon;		
    COORD_DM     lat;
    GFLOAT       height;
}               POINT_DM;

/***************************************************************
@    POINT_DMS
****************************************************************
A point id deg, min, sec
*/
typedef struct
{
    COORD_DMS       lon;
    COORD_DMS       lat;
}               POINT_DMS;

/*
 * Description:
 */

/***************************************************************
@    POINT_GEOREF
****************************************************************
A point in GEOREF
*/
typedef struct
{
    char            maj_zone[2];
    char            min_zone[2];
    GFLOAT          northing;
    GFLOAT          easting;
    GFLOAT          height;
}               POINT_GEOREF;

/*
 * Description:
 */


/***************************************************************
@    POINT_MGRS
****************************************************************
A point in Military Grid reference System
This is struct is to be determined
*/
typedef struct
{
    char            mgrs[30];
    GFLOAT          height;
}               POINT_MGRS;

/*
 * Description:
 */

/***************************************************************
@    POINT_UTM
****************************************************************
A point in Universal Transverse Mercator projection
*/
typedef struct
{
    int             zone;
    GFLOAT          northing;
    GFLOAT          easting;
    GFLOAT          height;
}               POINT_UTM;

/*
 * Description:
 */

/***************************************************************
@    POLAR_PARAMS
****************************************************************
Parameters for the ADRG Polar projection
*/
typedef struct
{
    GFLOAT          lat0;
    GFLOAT          lon0;
    GFLOAT          brv;
}               POLAR_PARAMS;

/*
 * Description:
 */

/***************************************************************
@  POINT_TYPE
****************************************************************
*/
typedef struct
{
   POINT_DD    rad_gp;
   POINT_DD    gp;
   POINT_UTM   utm;
   POINT_MGRS  mgrs;
   char        err_message[20];
   BOOLEAN     invalid_mgrs;

} POINT_TYPE;
/*
 * Description:
 */


/***************************************************************
@    POINT_UNITS
****************************************************************
Union of available types of point-oriented user units.
*/
typedef union
{
    POINT_DD        gpoint;  /* changed from MUSE structure */
    POINT_DM        dm;      /* added to MUSE structure */
    POINT_DMS       dms;
    POINT_UTM       utm;
    POINT_MGRS      mgrs;
    POINT_GEOREF    georef;
}               POINT_UNITS;
/***
Description: Used in the conversion functions for geographic units.
***/

/***************************************************************
@    POINT_ENTRY
****************************************************************
Data needed when getting a new user point
*/

typedef struct
{
    char            prompt_str[80];   
    POINT_UNITS     user_pt;
    BOOLEAN         user_cancelled;
}               POINT_ENTRY;

/*
 * Description: Data that is needed by or returned by the screen that prompts
 * user to enter a new data point.
 */

/***************************************************************
@    PROJ_DIR
****************************************************************
Map projection direction
*/
typedef enum
{
    PROJ_FORWARD,
    PROJ_REVERSE
}
                PROJ_DIR;

/*
 * Description: PROJ_FORWARD tells map_it() to convert from grographic
 * coordinates to PHIGS graphics coordinates. PROJ_REVERSE tells map_it() to
 * convert from PHIGS graphics coordinates back to geographic coordinates.
 */

/***************************************************************
@    PROJ_NAME_LEN
****************************************************************
Limit on length of a projection name
*/
#define PROJ_NAME_LEN 30

/*
 * Description:
 */


/***************************************************************
@    PROJECTION
****************************************************************
Map projections
*/
typedef enum
{
    PROJ_UNDEFINED,
    PROJ_ARC,
    PROJ_LAMBERT1,
    PROJ_LAMBERT2,
    PROJ_LINEAR,
    PROJ_MERCATOR,
    PROJ_UPS,
    PROJ_UTM,
    PROJ_TS,
    PROJ_GRAPH,
    PROJ_POLAR
}
                PROJECTION;

/*
 * Projections to be added in the future: PROJ_BONNE, PROJ_MOLLWEIDE,
 * PROJ_ORTHOGRAPHIC, PROJ_STEREOGRAPHIC, PROJ_TRANSVERSE_MERCATOR,
 * 
 * Description:
 */

/***************************************************************
@    SCALE
****************************************************************
Scale is stored internally as SCALE_RECIPROCAL.
*/
typedef enum
{
    SCALE_RECIPROCAL,
    SCALE_IN_PER_DEG
}
                SCALE;

/*
 * Description:
 */

/***************************************************************
@    STEREO_PARAMS
****************************************************************
Stereographic map projection parameters
*/
typedef struct
{
    GFLOAT          dummy;
}               STEREO_PARAMS;

/*
 * Description:
 */

/***************************************************************
@    TRANSM_PARAMS
****************************************************************
Transverse Mercator map projection parameters
*/
typedef struct
{
    long            izone;
}               TRANSM_PARAMS;
/*
 * Description:
 */

/***************************************************************
@    TS_PARAMS
****************************************************************
Tessellated Spheroid map projection parameters
*/
typedef struct
{
    short           ts_zone;
    short           ts_scale;
}               TS_PARAMS;

/*
 * Description:
 */


/***************************************************************
@    UNITS_TYPE
****************************************************************
List of available types of units.
*/
typedef union
{
    GFLOAT          dd2;
    COORD_DMS       dms;
    POINT_UTM       utm;
    POINT_MGRS      mgrs;
    POINT_GEOREF    georef;
    GFLOAT          mn;
    GFLOAT          sec;
}               UNITS_TYPE;

/*
 * Description:
 */

/***************************************************************
@    VER_DATUM
****************************************************************
*/
typedef enum
{
    UDEFINED,
    VD_ELIPSOID,
    VD_MSL
}
                VER_DATUM;

/*
 * Description:
 */

/***************************************************************
@    VER_DISTANCE
****************************************************************
Vertical distance is stored internally in meters.
*/
typedef enum
{
    VER_METERS,
    VER_FATHOMS,
    VER_FEET
}
                VER_DISTANCE;

/*
 * Description:
 */

/***************************************************************
@    ARC_PARMS
****************************************************************
*/
typedef struct
{
    long            scale;
    POINT_DD          origin;
    ARCPOINT        ul;
    ARCPOINT        lr;
    int             tile_size;
    int             zone;
}               ARC_PARMS;
/*
 * Description:
 */

/***************************************************************
@    DATUM
****************************************************************
Structure to contain the horizontal and vertical datums
*/
typedef struct
{
    HOR_DATUM       hor_datum;
    VER_DATUM       ver_datum;
}               DATUM;

/*
 * Description:
 */


/***************************************************************
@    MAP_POINT
****************************************************************
Data needed when getting a new user point
*/
typedef struct
{
    char            prompt_str[80];
    /* char *user_pt_str[40]; */
    POINT_UNITS     user_pt;
    POINT_DD          dd_pt;
}               MAP_POINT;

/*
 * Description: Data that is either needed by the dialog box that prompts
 * user to enter a new data point, or is returned by the box.
 */

/***************************************************************
@    PARAMS
****************************************************************
The union of projection-specific parameters
*/
typedef union
{
    LINEAR_PARAMS   linear_params;
    LAMBER_PARAMS   lamber_params;
    MERCAT_PARAMS   mercat_params;
    TRANSM_PARAMS   transm_params;
    ARC_PARMS       arc_params;
    TS_PARAMS       ts_params;
    POLAR_PARAMS    polar_params;
}               PARAMS;

/*
 * Description:
 */


#endif    /* H_COORD */




