/*
 * Copyright 1996 Vectaport Inc.
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems, Cider Press
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifndef mapcmds_h
#define mapcmds_h

#include <OverlayUnidraw/ovcmds.h>

#include <Unidraw/Commands/command.h>

extern "C" {
#include <projects.h>
}

class ACE_SOCK_Stream;
class GeoPointDialog;
class OpenFileChooser;

class CameraPositionRecord {
public:
    CameraPositionRecord() {}
    void setposition(float px, float py, float fx, float fy) {
        _px = px; _py = py; _fx = fx; _fy = fy;
    }
    float px() { return _px; }
    float py() { return _py; }
    float fx() { return _fx; }
    float fy() { return _fy; }
protected:
    float _px, _py, _fx, _fy;
};

class SendCameraCmd : public Command {
public:
    SendCameraCmd(float px, float py, float fx, float fy);
    virtual void Execute();
    virtual boolean Reversible() { return false; }
protected:
    CameraPositionRecord _pos;

#ifdef HAVE_ACE
    static ACE_SOCK_Stream* _socket;
#endif
};

#if 0
class OpenDemViewCmd : public Command {
public:
  OpenDemViewCmd(ControlInfo*);
  OpenDemViewCmd(Editor* =nil);
  virtual void Execute();
  virtual boolean Reversible() { return false; }
protected:
};
#endif

class MapOpenCmd : public OvOpenCmd {
public:
    MapOpenCmd(ControlInfo*, OpenFileChooser* = nil);
    MapOpenCmd(Editor* = nil, OpenFileChooser* = nil);

    virtual void Execute();

    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
};

class MapShowCmd : public Command {
public:
    MapShowCmd(ControlInfo*, boolean use_editor =true, boolean hidden =false);
    MapShowCmd(Editor* = nil, boolean use_editor =true, boolean hidden =false);

    virtual void Execute();
    virtual boolean Reversible();

    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    const char* GetLibName() { return _libname; }
    const char* GetCovName() { return _covname; }
    const char* GetFClassName() { return _fcname; }
    PJ* GetProjection() { return _proj; }
    boolean UseEditor() { return _use_ed; }
    boolean Hidden() { return _hidden; }
  int GetTileId() { return _tileid; }

    void SetLibName(const char*);
    void SetCovName(const char*);
    void SetFClassName(const char*);
    void SetUseEditor(boolean);
    void SetHidden(boolean);
    void SetMag(float);
    void SetProjection(PJ* p) { _proj = p; }
  void SetTileId(int i) { _tileid = i; }
protected:
    boolean _hidden;
    boolean _use_ed;
    char* _libname;
    char* _covname;
    char* _fcname;
    float _mag;
    PJ* _proj;
    int _tileid;
};

class MapShowAllCmd : public MapShowCmd {
public:
    MapShowAllCmd(ControlInfo*, boolean use_editor = false, boolean hidden =false);
    MapShowAllCmd(Editor* = nil, boolean use_editor = false, boolean hidden =false);

    virtual void Execute();

    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
};

class MapHideCmd : public Command {
public:
    MapHideCmd(ControlInfo*, boolean use_editor =true);
    MapHideCmd(Editor* = nil, boolean use_editor =true);

    virtual void Execute();
    virtual boolean Reversible();

    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    const char* GetLibName() { return _libname; }
    const char* GetCovName() { return _covname; }
    const char* GetFClassName() { return _fcname; }
    boolean UseEditor() { return _use_ed; }

    void SetLibName(const char*);
    void SetCovName(const char*);
    void SetFClassName(const char*);
    void SetUseEditor(boolean);
protected:
    boolean _use_ed;
    char* _libname;
    char* _covname;
    char* _fcname;
};

class MapCenterCmd : public Command {
public:
  MapCenterCmd(ControlInfo*);
  MapCenterCmd(Editor*);

  virtual boolean Reversible() { return false; }
  virtual void Execute();
  void CenterLonLat(float lon, float lat);
protected:
  float _lon, _lat;
};

class ContourCmd : public Command {
public:
    ContourCmd(ControlInfo*);
    ContourCmd(Editor* = nil);

    virtual void Execute();
    virtual boolean Reversible();

    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
};

class StartBlinkingCmd : public Command {
public:
    StartBlinkingCmd(ControlInfo*);
    StartBlinkingCmd(Editor* = nil);
    virtual Command* Copy();

    virtual void Execute();
};

class StopBlinkingCmd : public Command {
public:
    StopBlinkingCmd(ControlInfo*);
    StopBlinkingCmd(Editor* = nil);
    virtual Command* Copy();

    virtual void Execute();
};

#endif
